﻿using System;
using System.Collections.Generic;
using System.Linq;
using Newtonsoft.Json;

namespace DeviceModelLib
{
    public enum DeviceStatus
    {
        NoData,
        Ok,
        Warning,
        Error,
    }

    public class Measurement
    {
        public int DeviceId { get; set; }
        public float Value { get; set; } = float.NaN;
        public DateTime Timestamp { get; set; } = DateTime.Now;
        public DeviceStatus Status { get; set; } = DeviceStatus.NoData;      
        public string StatusAsString
        {
            get { return Status.ToString(); }
        }
    }

    public class BaseInfo
    {
        public int Id { get; set; }
        public string Name { get; set; }
        public string Description { get; set; }
    }

    public class DeviceTypeInfo : BaseInfo
    {
        public string Units { get; set; }
        public float Min { get; set; }
        public float Max { get; set; }
        public string TypeSpecificInfo { get; set; }
    }

    public class DeviceInfo : BaseInfo
    {
        public int TypeId { get; set; }
        public int GroupId { get; set; }
        public string DeviceSpecificInfo { get; set; }
        
        public static DeviceInfo FromJson(string json)
        {
            return JsonConvert.DeserializeObject<DeviceInfo>(json);
        }

        public override string ToString()
        {
            return $"{Id}, {TypeId}, {Name}, {Description}";
        }
    }

    public class GroupInfo : BaseInfo
    {
    }
}
