import { Component } from '@angular/core';
import { CommunicationService, DeviceMeasurement } from '../providers/communication.service';
import { ChartComponent } from '../chart/chart.component';

@Component({
	selector: 'devices',
	providers: [CommunicationService],
	templateUrl: 'devices.component.html',
	styleUrls: ['../app.component.css']
})
export class DevicesComponent {

	groupAppearance: Array<GroupAppearance> = [];
	private imagesUrl: Array<string> = ['fa fa-caret-right', 'fa fa-caret-down'];
	private isImagesInitialized = false;
	private mapChart: Map<number, ChartComponent>;
	private static currentThis: DevicesComponent;
	isShown: Boolean;

	constructor(private cs: CommunicationService) {
		CommunicationService.comm.selectComponentSubject.asObservable().subscribe(result => this.isShown = result == 'devices');
		CommunicationService.comm.receiveMeasurementSubject.asObservable().subscribe(() => this.receivedMeasurementCallback());
		this.mapChart = new Map<number, ChartComponent>();
		this.isShown = false;
		DevicesComponent.currentThis = this;
	}
	
	receivedMeasurementCallback() {
		this.obtainGroupsStatus();
	}

	getData() {	
		if (!this.isImagesInitialized && CommunicationService.comm.groups && CommunicationService.comm.groups.length > 0) {
			for (var i = 0; i < CommunicationService.comm.groups.length; i++) {
				var ga = new GroupAppearance();
				ga.image = this.imagesUrl[0];
				ga.imageIndex = 0;
				this.groupAppearance.push(ga);
			}
			this.isImagesInitialized = true;
		}

		return {
			groups: CommunicationService.comm.groups,
			deviceTypes: CommunicationService.comm.deviceTypes,
			devices: CommunicationService.comm.devices
		}
	}

	getAllDevicesInGroup(groupId) {
		return CommunicationService.comm.devices.filter(d => d.groupId == groupId);
	}

	getImage(groupIndex: number) {
		return this.groupAppearance[groupIndex].image;
	}

	toggleExpansion(groupIndex: number) {
		var ga = this.groupAppearance[groupIndex];
		ga.imageIndex = (ga.imageIndex + 1) % 2;
		ga.image = this.imagesUrl[ga.imageIndex];
		ga.isExpanded = ga.imageIndex == 1;
	}

	getLastMeasurementStatus(deviceId: number): number {
		var lastMeasurement = CommunicationService.comm.getLastMeasurementByDeviceId(deviceId);
		return lastMeasurement ? lastMeasurement.status : 0;
    }

	private arrStatus  = [
        'NoData',
        'OK',
        'Warning',
        'Error',
	];
	
	private arrBigGlyph  = [
        'fa fa-minus-circle',
        'fa fa-check-circle',
        'fa fa-question-circle',
        'fa fa-exclamation-circle',
	];
	
	private arrGlyph  = [
        'fa fa-minus',
        'fa fa-check',
        'fa fa-question',
        'fa fa-exclamation',
    ];

    public statusAsString(status: number): string {
        return this.arrStatus[status];
	}
	
	public statusAsBigGlyph(status: number): string {
        return this.arrBigGlyph[status];
	}
	
	public statusAsGlyph(status: number): string {
        return this.arrGlyph[status];
    }

	private obtainGroupsStatus() {	
		var co = CommunicationService.comm;	
		if (co && co.groups && co.groups.length > 0) {
						
			// Overall groups status indicator
			for (var i = 0; i < co.groups.length; i++) {
				var group = co.groups[i];
				var ga = this.groupAppearance[i];
				ga.groupStatus = 0;
				ga.groupStatusAsString = 'NoData';
				
				for (var j = 0; j < co.devices.length; j++) {
					var device = co.devices[j];
					let lastMeasurement: DeviceMeasurement = co.getLastMeasurementByDeviceId(device.id);
					if (lastMeasurement) {
						var status = lastMeasurement.status;
						if (group.id == device.groupId && status >= ga.groupStatus) {
							ga.groupStatus = status;
							ga.groupStatusAsString = this.statusAsString(status);
						}
					}
				}
			}
		}		
	}

	indicateCurrentDevice(deviceId: number) {
		var chart = this.mapChart.get(deviceId);
		if (!chart) {
			var deviceType = CommunicationService.comm.getTypeByDeviceId(deviceId);
			if (deviceType) {
				this.mapChart.set(deviceId, new ChartComponent(deviceId,));
				chart = this.mapChart.get(deviceId);
				chart.createChart(deviceType.min, deviceType.max, deviceType.units);
			}
		}
	}
}

export class GroupAppearance {
	groupId: number;
	image: string;
	imageIndex: number;
	groupStatus: number;
	groupStatusAsString: string;
	isExpanded: boolean = false;
}
